#if !defined(AFX_UTP__H__INCLUDED_)
#define AFX_UTP__H__INCLUDED_


/*
 * Copyright (c) 2010-2012 Rozhuk Ivan <rozhuk.im@gmail.com>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 */



#define PACKET_SIZE 300


#pragma pack(push, 1)

typedef struct utp_pkt_s { 		/* offset - PacketFormat */
	uint32_t	connid;		/* 00 connection ID */
	uint32_t	tv_sec;		/* 04  */
	uint32_t	tv_usec;	/* 08  */
	uint32_t	reply_micro;	/* 12  */
	uint8_t		windowsize;	/* 16 receive window size in PACKET_SIZE chunks */
	uint8_t		ext;		/* 17 Type of the first extension header */
	uint8_t		flags;		/* 18 Flags */
	uint16_t	seq_nr;		/* 19 Sequence number */
	uint16_t	ack_nr;		/* 21 Acknowledgment number */
					/* 23 ext/data */
} utp_pkt_t, *utp_pkt_p; /* 23 bytes */
/*
 * syn - 14 bytes
 * 'udp[17] = 2 and udp[18] = 4 and udp[21:2] = 0 and udp[23] = 0 and udp[24] = 8 and udp[25:4] = 0 and udp[29:4] = 0'
 * 41 = udp hdr len (8) + upd pkt data len
 * upd header included:
 * '(udp[4:2] = 41 and udp[25:2] = 0x0204 and udp[29:4] = 0x00000008 and udp[33:4] = 0 and udp[37:4] = 0)'
 */
/*
 * rst - 4 bytes
 * 'udp[17] = 0 and udp[18] = 3'
 * 31 = udp hdr len (8) + upd pkt data len
 * upd header included:
 * '(udp[4:2] = 31 and udp[25:2] = 0x0003)'
 */


typedef struct utp_pkt_v1_s { 		/* offset - PacketFormatV1 */
	uint8_t		version:4;	/* 00 protocol version */
	uint8_t		type:4;		/* 00 type (formerly flags) */
	uint8_t		ext;		/* 01 Type of the first extension header */
	uint16_t	connid;		/* 02 connection ID */
	uint32_t	tv_usec;	/* 04  */
	uint32_t	reply_micro;	/* 08  */
	uint32_t	windowsize;	/* 12 receive window size in bytes */
	uint16_t	seq_nr;		/* 16 Sequence number */
	uint16_t	ack_nr;		/* 18 Acknowledgment number */
					/* 20 ext/data */
} utp_pkt_v1_t, *utp_pkt_v1_p; /* 20 bytes */
/*
 * syn - 14 bytes
 * 'udp[0] & 0x0f = 1 and udp[0] & 0xf0 = 0x40 and udp[1] = 2 and udp[18:2] = 0 and udp[20] = 0 and udp[21] = 8 and udp[22:4] = 0 and udp[26:4] = 0'
 * (udp[0] & 0x0f = 1 and udp[0] & 0xf0 = 0x40) => udp[0] = 0x41
 * 38 = udp hdr len (8) + upd pkt data len
 * upd header included:
 * '(udp[4:2] = 38 and udp[8:2] = 0x4102 and udp[26:4] = 0x00000008 and udp[30:4] = 0 and udp[34:4] = 0)'
 */
/*
 * rst - 4 bytes
 * 'udp[0] & 0x0f = 1 and udp[0] & 0xf0 = 0x30 and udp[1] = 0'
 * (udp[0] & 0x0f = 1 and udp[0] & 0xf0 = 0x30) => udp[0] = 0x31)
 * 28 = udp hdr len (8) + upd pkt data len
 * upd header included:
 * '(udp[4:2] = 28 and udp[8:2] = 0x3100)'
 */

#define UTP_VERS_COUNT		2
#define UTP_MIN_PKT_SIZE	sizeof(utp_pkt_v1_t) /* = 20 */



enum {
	ST_DATA		= 0,	/* Data packet. */
	ST_FIN		= 1,	/* Finalize the connection. This is the last packet. */
	ST_STATE	= 2,	/* State packet. Used to transmit an ACK with no data. */
	ST_RESET	= 3,	/* Terminate connection forcefully. */
	ST_SYN		= 4,	/* Connect SYN. */
	ST_NUM_STATES		/* Used for bounds checking. */
};


const char *flagnames[] = {
	"ST_DATA",
	"ST_FIN",
	"ST_STATE",
	"ST_RESET",
	"ST_SYN",
	NULL
};



/* non standart: removed PacketFormat[V1] */
typedef struct utp_pkt_ext_s {
	uint8_t	ext_next;
	uint8_t	ext_len;
	/* optional data */
} utp_pkt_ext_t, *utp_pkt_ext_p;


enum {
	EXT_NO_EXT	= 0,	/* no extension */
	EXT_SEL_ACKS	= 1,	/* Selective acks: Its payload is a bitmask of at least 32 bits, in multiples of 32 bits */
	EXT_BITS	= 2,	/* Extension bits: It is an 8 byte bitmask where each bit specifies support for a specific feature */
	EXT_MAX
};

const uint32_t EXT_MAX_COUNT[EXT_MAX] = {
	1, /* EXT_NO_EXT */
	1, /* EXT_SEL_ACKS */
	1  /* EXT_BITS */
};


/*
ST_DATA & EXT_NO_EXT
ST_FIN & EXT_NO_EXT
ST_STATE & (EXT_NO_EXT | EXT_SEL_ACKS)
ST_RESET & EXT_NO_EXT
ST_SYN & (EXT_NO_EXT | EXT_BITS)
*/

#pragma pack(pop)






#ifndef NTOHS

static __inline u_short USHORT_FLIP(u_short usIn)
{	
return(((usIn<<8) | (usIn>>8)));
}
#define NTOHS(in)   USHORT_FLIP(in)
#define HTONS(in)   USHORT_FLIP(in)

#endif



#ifndef NTOHL

static __inline u_long ULONG_FLIP(u_long ulIn)
{
return((((ulIn<<8) & 0x00ff0000) | (ulIn<<24) | ((ulIn>>8) & 0x0000ff00) | (ulIn>>24)));
}

#define NTOHL(in) ULONG_FLIP(in)
#define HTONL(in) ULONG_FLIP(in)

#endif


//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
//////////////////////////////Packet vars/////////////////////////////////
//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////

static __inline uint32_t
uTPGetConnID(int pkt_ver, u_char *buf) { /* connection ID */

	switch (pkt_ver) {
	case 0:
		return(NTOHL(((utp_pkt_p)buf)->connid));
	case 1:
		return(NTOHS(((utp_pkt_v1_p)buf)->connid));
	}
return(0);
}

static __inline uint32_t
uTPGetTVUsec(int pkt_ver, u_char *buf) { /*  */

	switch (pkt_ver) {
	case 0:
		return(((NTOHL(((utp_pkt_p)buf)->tv_sec) * 1000000) + NTOHL(((utp_pkt_p)buf)->tv_usec)));
	case 1:
		return(NTOHL(((utp_pkt_v1_p)buf)->tv_usec));
	}
return(0);
}


static __inline uint32_t
uTPGetReplyMicro(int pkt_ver, u_char *buf) { /*  */
	uint32_t uiRet;

	switch (pkt_ver) {
	case 0:
		uiRet = NTOHL(((utp_pkt_p)buf)->reply_micro);
		uiRet = ((uiRet == ((uint32_t)~0)) ? 0 : uiRet);
		break;
	case 1:
		uiRet = NTOHL(((utp_pkt_v1_p)buf)->reply_micro);
		uiRet = ((uiRet == ((uint32_t)~0)) ? 0 : uiRet); /* ? - violete specification */
		break;
	default:
		uiRet = 0;
	}
return(uiRet);
}

static __inline uint32_t
uTPGetWindowSize(int pkt_ver, u_char *buf) { /* receive window size */

	switch (pkt_ver) {
	case 0:
		return((((utp_pkt_p)buf)->windowsize * PACKET_SIZE)); /* receive window size in PACKET_SIZE chunks */
	case 1:
		return((((utp_pkt_v1_p)buf)->windowsize)); /* receive window size in bytes */
	}
return(0);
}


static __inline uint16_t
uTPGetSeqNr(int pkt_ver, u_char *buf) { /* Sequence number */

	switch (pkt_ver) {
	case 0:
		return(NTOHS(((utp_pkt_p)buf)->seq_nr));
	case 1:
		return(NTOHS(((utp_pkt_v1_p)buf)->seq_nr));
	}
return(0);
}

static __inline uint16_t
uTPGetAckNr(int pkt_ver, u_char *buf) { /* Acknowledgment number */

	switch (pkt_ver) {
	case 0:
		return(NTOHS(((utp_pkt_p)buf)->ack_nr));
	case 1:
		return(NTOHS(((utp_pkt_v1_p)buf)->ack_nr));
	}
return(0);
}


static __inline uint8_t
uTPGetFlagsType(int pkt_ver, u_char *buf) { /* type (formerly flags) = ST_* */

	switch (pkt_ver) {
	case 0:
		return(((utp_pkt_p)buf)->flags);
	case 1:
		return(((utp_pkt_v1_p)buf)->type);
	}
return(0);
}


static __inline uint8_t
uTPGetExt(int pkt_ver, u_char *buf) { /* Type of the first extension header */

	switch (pkt_ver) {
	case 0:
		return(((utp_pkt_p)buf)->ext);
	case 1:
		return(((utp_pkt_v1_p)buf)->ext);
	}
return(0);
}
//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
//////////////////////////////Packet process//////////////////////////////
//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////
static __inline int
uTPIsValidExt(uint8_t ext, u_char *buf, uint32_t buf_size) { /* Verify that the packet extensions are valid. */
	u_char *buf_max;
	utp_pkt_ext_t *pkt_ext;
	uint32_t ext_count[EXT_MAX];

	if (ext == EXT_NO_EXT) /* no extensions - valid, no size check here */
		return(1);
	if (ext >= EXT_MAX || buf_size < sizeof(utp_pkt_ext_t)) /* buff to small to contain any extensions or invalid ext type number */
		return(0);

	pkt_ext = (utp_pkt_ext_p)buf;
	buf_max = (buf + buf_size);
	bzero(&ext_count, sizeof(ext_count));
	do {
		if (ext >= EXT_MAX || buf_max < (((u_char *)pkt_ext) + sizeof(utp_pkt_ext_t))) /* bad ext type num / out of buff range */
			return(0);

		/* we can read ext_len and ext_next - they in buff range */
		if (buf_max < (((u_char*)pkt_ext) + sizeof(utp_pkt_ext_t) + pkt_ext->ext_len)) /* bad ext len? (out of pkt range) */
			return(0);
		/* check etx len */
		switch (ext) {
		case EXT_NO_EXT:
			if (pkt_ext->ext_len != 0) /* bad end of extensions */
				return(0);
			break;
		case EXT_SEL_ACKS:
			if ((pkt_ext->ext_len & 3) != 0) /* bad Selective acks extension, must be multiple 4 */
				return(0);
			break;
		case EXT_BITS:
			if (pkt_ext->ext_len != 8) /* bad bits extension */
				return(0);
			break;
		}
		/* check max ext cont */
		ext_count[ext] ++;
		if (EXT_MAX_COUNT[ext] < ext_count[ext])
			return(0); /* too many extensions of this type */
		/* move to next ext */
		ext = pkt_ext->ext_next;
		pkt_ext = (utp_pkt_ext_p)(((u_char*)pkt_ext) + sizeof(utp_pkt_ext_t) + pkt_ext->ext_len);
	} while (ext != EXT_NO_EXT);

return(1);
}



static __inline int
uTPIsPacketv0(u_char *buf, uint32_t buf_size) {
	utp_pkt_p utp_pkt = (utp_pkt_p)buf;

	if (buf_size < sizeof(utp_pkt_t))
		return(0);
	if (utp_pkt->flags >= ST_NUM_STATES)
		return(0);

return(uTPIsValidExt(utp_pkt->ext, (buf + sizeof(utp_pkt_t)), (buf_size - sizeof(utp_pkt_t))));
}


static __inline int
uTPIsPacketv1(u_char *buf, uint32_t buf_size) {
	utp_pkt_v1_p utp_pkt_v1 = (utp_pkt_v1_p)buf;

	if (buf_size < sizeof(utp_pkt_v1_t))
		return(0);
	if (utp_pkt_v1->version != 1 || utp_pkt_v1->type >= ST_NUM_STATES)
		return(0);

return(uTPIsValidExt(utp_pkt_v1->ext, (buf + sizeof(utp_pkt_v1_t)), (buf_size - sizeof(utp_pkt_v1_t))));
}

static __inline int
uTPGetPacketVersion(u_char *buf, uint32_t buf_size) {

	if (uTPIsPacketv1(buf, buf_size))
		return(1);
	if (uTPIsPacketv0(buf, buf_size))
		return(0);
return(-1);
}



static __inline uint32_t
uTPGenRSTPacket(int pkt_ver, uint32_t connid, uint16_t ack_nr, uint16_t seq_nr, u_char *buf, uint32_t buf_size) {
	utp_pkt_p utp_pkt;
	utp_pkt_v1_p utp_pkt_v1;

	switch (pkt_ver) {
	case 0:
		if (buf_size < sizeof(utp_pkt_t))
			return(0);

		utp_pkt = (utp_pkt_p)buf;
		bzero(buf, sizeof(utp_pkt_t));

		utp_pkt->connid = HTONL(connid);
		utp_pkt->ack_nr = HTONS(ack_nr);
		utp_pkt->seq_nr = HTONS(seq_nr);
		utp_pkt->flags = ST_RESET;
		//utp_pkt->ext = 0;
		//utp_pkt->windowsize = 0;

		return(sizeof(utp_pkt_t));
	case 1:
		if (buf_size < sizeof(utp_pkt_v1_t))
			return(0);

		utp_pkt_v1 = (utp_pkt_v1_p)buf;
		bzero(buf, sizeof(utp_pkt_v1_t));

		utp_pkt_v1->version = 1;
		utp_pkt_v1->type = ST_RESET;
		//utp_pkt_v1->ext = 0;
		utp_pkt_v1->connid = HTONS(connid);
		utp_pkt_v1->ack_nr = HTONS(ack_nr);
		utp_pkt_v1->seq_nr = HTONS(seq_nr);
		//utp_pkt_v1->windowsize = 0;

		return(sizeof(utp_pkt_v1_t));
	}
return(0);
}







#endif // AFX_UTP__H__INCLUDED_

